// SocketWrench 10
// Copyright 2023 Catalyst Development Corporation
// All rights reserved
// 
// This file is licensed to you pursuant to the terms of the
// product license agreement included with the original software,
// and is protected by copyright law and international treaties.
// Unauthorized reproduction or distribution may result in severe
// criminal penalties.
// 

using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Threading; 

namespace EchoServer
{
    public class SessionClass  
    { 
        public ServerForm Server; 
        public SocketTools.SocketWrench Socket; 
        public int Handle; 
        private bool Terminated; 
        
        public void ThreadMain() 
        { 
			int nThreadId = 0; 
            string strThreadName = null; 
            string strBuffer = null; 
            int nResult = 0; 
            
            nThreadId = System.AppDomain.GetCurrentThreadId(); 
            strThreadName = "Thread " +  nThreadId.ToString("X") + " (" + Thread.CurrentThread.Name + ")"; 
            
            //  Create an instance of the class
            Socket = new SocketTools.SocketWrench(); 
            Socket.Initialize(); 
            
            if (! Socket.Accept( Handle )) 
            { 
                Server.WriteLine( strThreadName + " unable to accept connection, error " + Socket.LastError ); 
                return; 
            } 
            
            if ( Server.RegisterClient( this, Thread.CurrentThread ) == false ) 
            { 
                Socket.Disconnect(); 
                Server.WriteLine( strThreadName + " unable to accept connection, too many active clients" ); 
                return; 
            } 
            
            Server.WriteLine( strThreadName + " accepted connection from " + Socket.PeerAddress ); 
            
            while (! Terminated ) 
            { 
                if ( Socket.IsReadable ) 
                { 
                    nResult = Socket.Read( ref strBuffer, 1024 ); 
                    if ( nResult == 0 ) 
                    { 
                        Server.WriteLine( strThreadName + " connection closed by client" ); 
                        break;
                    } 
                    else if ( nResult == -1 ) 
                    { 
                        Server.WriteLine( strThreadName + " socket read failed, error " + Socket.LastError ); 
                    } 
                    
                    nResult = Socket.Write( strBuffer, nResult ); 
                    if ( nResult == -1 ) 
                    { 
                        Server.WriteLine( strThreadName + " socket write failed, error " + Socket.LastError ); 
                        break;
                    } 
                } 
            } 
            
            Socket.Disconnect(); 
            Server.UnregisterClient( this ); 
            return; 
        } 
        
        public void Terminate() 
        { 
            Terminated = true; 
        } 
    } 
} 
