' SocketTools 10 Build 1468
' Copyright 2023 Catalyst Development Corporation
' All rights reserved
'
' This file is licensed to you pursuant to the terms of the
' product license agreement included with the original software,
' and is protected by copyright law and international treaties.
' Unauthorized reproduction or distribution may result in severe
' criminal penalties.
'
Option Strict Off
Option Explicit On

Friend Class formMain
    Inherits System.Windows.Forms.Form

    Private WithEvents ftpServer As New SocketTools.FtpServer

    <System.STAThread()> Public Shared Sub Main()
        Application.EnableVisualStyles()
        Application.DoEvents()
        Application.Run(New formMain)
    End Sub

    Delegate Sub UpdateFormCallback(ByVal bEnable As Boolean)

    Private Sub UpdateForm(ByVal bEnable As Boolean)
        If cmdStart.InvokeRequired Then
            Dim d As New UpdateFormCallback(AddressOf UpdateForm)
            Me.Invoke(d, New Object() {bEnable})
        Else
            If ftpServer.IsActive Then
                cmdStart.Enabled = bEnable
                cmdSuspend.Enabled = bEnable
                cmdRestart.Enabled = bEnable
            Else
                cmdStart.Enabled = bEnable
                cmdSuspend.Enabled = False
                cmdRestart.Enabled = False
            End If
        End If
    End Sub

    Delegate Sub UpdateStatusCallback(ByVal strMessage As String)

    Private Sub UpdateStatus(ByVal strMessage As String)
        If lblStatus.InvokeRequired Then
            Dim d As New UpdateStatusCallback(AddressOf UpdateStatus)
            Me.Invoke(d, New Object() {strMessage})
        Else
            lblStatus.Text = strMessage
            lblClient.Text = ftpServer.ClientCount & " Clients"
        End If
    End Sub

    Private Sub formMain_Load(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        comboFormats.Items.Insert(0, "None")
        comboFormats.Items.Insert(1, "Common")
        comboFormats.Items.Insert(2, "Extended")
        comboFormats.SelectedIndex = 0
    End Sub

    Private Sub cmdStart_Click(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles cmdStart.Click

        If Not ftpServer.IsActive Then

            UpdateForm(False)

            ' Basic server configuration
            ftpServer.ServerName = Trim(editHostName.Text)
            ftpServer.ServerAddress = Trim(editLocalAddress.Text)
            ftpServer.ServerPort = CInt(Trim(editLocalPort.Text))
            ftpServer.Identity = Trim(editIdentity.Text)
            ftpServer.Directory = Trim(editDirectory.Text)

            ' Server limits
            ftpServer.MaxClients = CInt(editMaxClients.Text)
            ftpServer.MaxGuests = CInt(editMaxGuests.Text)
            ftpServer.IdleTime = CInt(editIdleTime.Text)
            ftpServer.AuthFail = CInt(editMaxLogins.Text)
            ftpServer.MinPort = CInt(editMinPort.Text)
            ftpServer.MaxPort = CInt(editMaxPort.Text)

            ' Server options
            ftpServer.Secure = CBool(chkSecure.CheckState)
            ftpServer.LocalUser = CBool(chkLocalUser.CheckState)
            ftpServer.MultiUser = CBool(chkMultiUser.CheckState)
            ftpServer.UnixMode = CBool(chkUnixMode.CheckState)
            ftpServer.LocalTime = CBool(chkLocalTime.CheckState)
            ftpServer.LockFiles = CBool(chkLockFiles.CheckState)
            ftpServer.Restricted = CBool(chkRestricted.CheckState)
            ftpServer.ReadOnly = CBool(chkReadOnly.CheckState)
            ftpServer.HiddenFiles = CBool(chkHidden.CheckState)

            ' Server logging
            ftpServer.LogFormat = comboFormats.SelectedIndex
            ftpServer.LogFile = Trim(editLogFile.Text)

            ' Start the server
            If ftpServer.Start() Then
                cmdStart.Text = "Stop"
                UpdateStatus("Server accepting connections on " & ftpServer.ServerAddress & ":" & ftpServer.ServerPort)
                UpdateForm(True)
            Else
                MsgBox(ftpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        Else
            UpdateForm(False)

            If ftpServer.Stop() Then
                cmdStart.Text = "Start"
            Else
                MsgBox(ftpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        End If
    End Sub

    Private Sub cmdSuspend_Click(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles cmdSuspend.Click
        If cmdSuspend.Text = "Suspend" Then
            UpdateForm(False)

            If ftpServer.Suspend() Then
                cmdSuspend.Text = "Resume"
                UpdateStatus("Server suspended accepting new client connections")
                UpdateForm(True)
            Else
                MsgBox(ftpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If

        Else
            UpdateForm(False)

            If ftpServer.Resume() Then
                cmdSuspend.Text = "Suspend"
                UpdateStatus("Server resumed accepting new client connections")
                UpdateForm(True)
            Else
                MsgBox(ftpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        End If

    End Sub

    Private Sub cmdRestart_Click(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles cmdRestart.Click

        UpdateForm(False)

        If ftpServer.Restart() Then
            cmdSuspend.Text = "Suspend"
            UpdateStatus("The server has been restarted")
            UpdateForm(True)
        Else
            MsgBox(ftpServer.LastErrorString, MsgBoxStyle.Exclamation)
            UpdateForm(True)
            Exit Sub
        End If

    End Sub

    Private Sub cmdClose_Click(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles cmdClose.Click

        If ftpServer.IsActive Then
            UpdateForm(False)

            If ftpServer.Stop() Then
                UpdateForm(True)
            Else
                MsgBox(ftpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        End If

        Me.Close()
    End Sub

    Private Sub ftpServer_OnAuthenticate(ByVal sender As Object, ByVal e As SocketTools.FtpServer.AuthenticateEventArgs) Handles ftpServer.OnAuthenticate
        Debug.WriteLine("Client " & e.ClientId & " authenticating as " & e.UserName)
    End Sub

    Private Sub ftpServer_OnCommand(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.CommandEventArgs) Handles ftpServer.OnCommand
        Debug.WriteLine("Client " & e.ClientId & " sent command " & e.Command & " [" & e.Parameters & "]")
        Debug.WriteLine("Virtual Path: " & ftpServer.VirtualPath)
        Debug.WriteLine("Local Path:   " & ftpServer.LocalPath)

        If e.Command = "SITE" Then
            If UCase(e.Parameters) = "TIME" Then
                ftpServer.SendResponse(e.ClientId, 200, "The local time is " & Today & " " & TimeOfDay)
            End If
        End If

    End Sub

    Private Sub ftpServer_OnConnect(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.ConnectEventArgs) Handles ftpServer.OnConnect
        Debug.WriteLine("Client " & e.ClientId & " connected from " & e.ClientAddress)
        UpdateStatus("Client " & e.ClientId & " connected from " & e.ClientAddress)
    End Sub

    Private Sub ftpServer_OnDisconnect(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.DisconnectEventArgs) Handles ftpServer.OnDisconnect
        Debug.WriteLine("Client " & e.ClientId & " disconnected")
        UpdateStatus("Client " & e.ClientId & " disconnected from the server")
    End Sub

    Private Sub ftpServer_OnDownload(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.DownloadEventArgs) Handles ftpServer.OnDownload
        Debug.WriteLine("Client " & e.ClientId & " downloaded file " & e.FileName)
    End Sub

    Private Sub ftpServer_OnError(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.ErrorEventArgs) Handles ftpServer.OnError
        Debug.WriteLine("Client " & e.ClientId & " error " & e.Error & ": " & e.Description)
    End Sub

    Private Sub ftpServer_OnExecute(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.ExecuteEventArgs) Handles ftpServer.OnExecute
        Debug.WriteLine("Client " & e.ClientId & " executed " & e.Program & " (" & e.ExitCode & ")")
    End Sub

    Private Sub ftpServer_OnIdle(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles ftpServer.OnIdle
        Debug.WriteLine("All clients have disconnected, the server is idle")
    End Sub

    Private Sub ftpServer_OnLogin(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.LoginEventArgs) Handles ftpServer.OnLogin
        Debug.WriteLine("Client " & e.ClientId & " logged in as " & e.UserName & " (" & e.Directory & ")")
    End Sub

    Private Sub ftpServer_OnLogout(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.LogoutEventArgs) Handles ftpServer.OnLogout
        Debug.WriteLine("Client " & e.ClientId & " logged out as " & e.UserName)
    End Sub

    Private Sub ftpServer_OnResult(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.ResultEventArgs) Handles ftpServer.OnResult
        Debug.WriteLine("Client " & e.ClientId & " command " & e.Command & " completed (" & e.ResultCode & ")")
    End Sub

    Private Sub ftpServer_OnStart(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles ftpServer.OnStart
        Debug.WriteLine("Server has started accepting connections")
        UpdateStatus("Server accepting connections on " & ftpServer.ServerAddress & ":" & ftpServer.ServerPort)
        UpdateForm(True)
    End Sub

    Private Sub ftpServer_OnStop(ByVal eventSender As System.Object, ByVal e As System.EventArgs) Handles ftpServer.OnStop
        Debug.WriteLine("Server has stopped, all clients terminated")
        UpdateStatus("Server stopped accepting connections")
        UpdateForm(True)
    End Sub

    Private Sub ftpServer_OnTimeout(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.TimeoutEventArgs) Handles ftpServer.OnTimeout
        Debug.WriteLine("Client " & e.ClientId & " timed out after " & e.Elapsed & " seconds")
    End Sub

    Private Sub ftpServer_OnUpload(ByVal eventSender As System.Object, ByVal e As SocketTools.FtpServer.UploadEventArgs) Handles ftpServer.OnUpload
        Debug.WriteLine("Client " & e.ClientId & " uploaded file " & e.FileName)
    End Sub
End Class
