' SocketTools 10 Build 1468
' Copyright 2023 Catalyst Development Corporation
' All rights reserved
'
' This file is licensed to you pursuant to the terms of the
' product license agreement included with the original software,
' and is protected by copyright law and international treaties.
' Unauthorized reproduction or distribution may result in severe
' criminal penalties.
'
Option Strict Off
Option Explicit On

Friend Class MainForm
    Inherits System.Windows.Forms.Form
    Private WebStorage1 As New SocketTools.WebStorage()

    Private Sub UpdateForm(Optional ByVal bEnable As Boolean = True)
        If WebStorage1.IsOpened Then
            cmdOpen.Enabled = False
            cmdRefresh.Enabled = bEnable
            cmdUpload.Enabled = bEnable
            cmdDownload.Enabled = bEnable
            cmdDelete.Enabled = bEnable
            cmdClose.Enabled = bEnable
        Else
            cmdOpen.Enabled = True
            cmdRefresh.Enabled = False
            cmdUpload.Enabled = False
            cmdDownload.Enabled = False
            cmdDelete.Enabled = False
            cmdClose.Enabled = False
            listLabels.Items.Clear()
        End If
    End Sub

    Private Sub MainForm_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        cmdOpen.Enabled = True
        cmdRefresh.Enabled = False
        cmdUpload.Enabled = False
        cmdDownload.Enabled = False
        cmdClose.Enabled = False
        listLabels.Items.Clear()
    End Sub

    Private Sub cmdClose_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdClose.Click
        WebStorage1.Close()
        UpdateForm(False)
    End Sub

    Private Sub cmdDelete_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdDelete.Click
        If WebStorage1.Delete(txtLabelName.Text) Then
            lblInfo.Text = "Deleted " & txtLabelName.Text & " from container"
            MsgBox("Deleted " & txtLabelName.Text, MsgBoxStyle.Information)
            cmdRefresh_Click(cmdRefresh, New System.EventArgs())
        Else
            MsgBox(WebStorage1.LastErrorString, MsgBoxStyle.Exclamation)
        End If
    End Sub

    Private Sub cmdDownload_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdDownload.Click
        UpdateForm(False)

        If WebStorage1.GetFile(txtLocalFile.Text, txtLabelName.Text) Then
            lblInfo.Text = "Downloaded " & txtLabelName.Text & " from container"
            MsgBox("Downloaded " & txtLabelName.Text, MsgBoxStyle.Information)
        Else
            MsgBox(WebStorage1.LastErrorString, MsgBoxStyle.Exclamation)
        End If

        UpdateForm(True)
    End Sub

    Private Sub cmdOpen_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdOpen.Click
        If WebStorage1.Open() Then
            UpdateForm(True)
            cmdRefresh_Click(cmdRefresh, New System.EventArgs())
        Else
            MsgBox(WebStorage1.LastErrorString, MsgBoxStyle.Exclamation)
            Exit Sub
        End If
    End Sub

    Private Sub cmdRefresh_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdRefresh.Click
        Dim nObjects As Integer

        listLabels.Items.Clear()
        nObjects = 0
        UpdateForm(False)

        If WebStorage1.FindFirst() Then
            Do
                nObjects = nObjects + 1
                listLabels.Items.Add(WebStorage1.ObjectLabel)
            Loop Until Not WebStorage1.FindNext()
        End If

        UpdateForm(True)
        lblInfo.Text = "Found " & nObjects & " objects in current container"
    End Sub

    Private Sub cmdUpload_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdUpload.Click
        UpdateForm(False)

        If WebStorage1.PutFile(txtLocalFile.Text, txtLabelName.Text) Then
            lblInfo.Text = "Uploaded " & txtLabelName.Text & " in container"
            MsgBox("Uploaded " & txtLabelName.Text, MsgBoxStyle.Information)
            cmdRefresh_Click(cmdRefresh, New System.EventArgs())
        Else
            MsgBox(WebStorage1.LastErrorString, MsgBoxStyle.Exclamation)
        End If

        UpdateForm(True)
    End Sub

    Private Sub listLabels_SelectedIndexChanged(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles listLabels.SelectedIndexChanged
        txtLabelName.Text = VB6.GetItemString(listLabels, listLabels.SelectedIndex)
    End Sub
End Class