Imports SocketTools.InternetServer

Public Class Form1

    Private WithEvents EchoServer As New SocketTools.InternetServer
    Private ClientTable As New Hashtable

    ' The following code demonstrates how to create a method to safely
    ' update the ListBox control from another thread. First, delcare a
    ' delegate that will be used with the Invoke method.

    Delegate Sub ListBoxCallback(ByVal strMessage As String)

    ' Create a public method that will update the ListBox control
    ' in the UI thread.

    Public Sub UpdateListBox(ByVal strMessage As String)
        If Me.ListBox1.InvokeRequired Then
            Dim dMethod As New ListBoxCallback(AddressOf UpdateListBox)
            Me.Invoke(dMethod, New Object() {strMessage})
        Else
            Me.ListBox1.Items.Add(strMessage)
            Me.ListBox1.SelectedIndex = Me.ListBox1.Items.Count - 1
            Me.ListBox1.ClearSelected()
        End If
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Initialize the InternetServer class
        If Not EchoServer.Initialize() Then
            MsgBox("Unable to initialize SocketWrench server", MsgBoxStyle.Exclamation)
            End
        End If
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        If EchoServer.Status = ServerStatus.serverInactive Then
            Dim strLocalAddress As String
            Dim nLocalPort As Integer
            Dim nMaxClients As Integer

            strLocalAddress = TextBox1.Text.Trim()
            nLocalPort = Integer.Parse(TextBox2.Text)
            nMaxClients = Integer.Parse(TextBox3.Text)

            If nLocalPort < 1 Or nLocalPort > 65535 Then
                MsgBox("Please specify a valid port number for the server", MsgBoxStyle.Exclamation)
                TextBox2.Focus()
                Exit Sub
            End If

            Button1.Enabled = False

            If EchoServer.Start(strLocalAddress, nLocalPort, nMaxClients) Then
                UpdateListBox("Server started listening for connections")
            Else
                MsgBox(EchoServer.LastErrorString, MsgBoxStyle.Exclamation)
                Button1.Enabled = True
                Exit Sub
            End If

            Button1.Text = "Stop"
            Button1.Enabled = True
        Else
            If EchoServer.Stop() Then
                UpdateListBox("Server stopped listening for connections")
            Else
                MsgBox(EchoServer.LastErrorString, MsgBoxStyle.Exclamation)
                Exit Sub
            End If

            Button1.Text = "Start"
        End If
    End Sub

    Private Sub EchoServer_OnAccept(ByVal sender As Object, ByVal e As SocketTools.InternetServer.AcceptEventArgs) Handles EchoServer.OnAccept
        UpdateListBox("Incoming client connection from " + e.ClientAddress + ":" + e.ClientPort.ToString())
    End Sub

    Private Sub EchoServer_OnConnect(ByVal sender As Object, ByVal e As SocketTools.InternetServer.ConnectEventArgs) Handles EchoServer.OnConnect
        ' Assign a unique name to the client session based on its client ID
        EchoServer.ClientName = "Client" + EchoServer.ClientId.ToString()

        ' Initialize an entry in the client hashtable
        SyncLock ClientTable.SyncRoot
            ClientTable.Add(EchoServer.ClientId, 0)
        End SyncLock

        UpdateListBox(EchoServer.ClientName + " connected to server - " + EchoServer.ClientHandle.Count.ToString)
    End Sub

    Private Sub EchoServer_OnDisconnect(ByVal sender As Object, ByVal e As SocketTools.InternetServer.DisconnectEventArgs) Handles EchoServer.OnDisconnect
        Dim nBytesTotal As Integer

        ' Get the total number of bytes that have been exchanged
        SyncLock ClientTable.SyncRoot
            nBytesTotal = ClientTable(EchoServer.ClientId)
        End SyncLock

        UpdateListBox(EchoServer.ClientName + " disconnected from server (" + nBytesTotal.ToString() + " bytes exchanged)")
    End Sub

    Private Sub EchoServer_OnError(ByVal sender As Object, ByVal e As SocketTools.InternetServer.ErrorEventArgs) Handles EchoServer.OnError
        UpdateListBox("Error " + e.Error.ToString() + " " + e.Description + " [" + e.Handle.ToString() + "]")
    End Sub

    Private Sub EchoServer_OnRead(ByVal sender As Object, ByVal e As SocketTools.InternetServer.ReadEventArgs) Handles EchoServer.OnRead
        Dim ioBuffer(4096) As Byte
        Dim nBytesRead As Integer
        Dim nBytesWritten As Integer
        Dim nBytesTotal As Integer

        ' Read some data that the client has sent to the server
        nBytesRead = EchoServer.Read(ioBuffer)

        If nBytesRead > 0 Then
            ' Echo the data back to the client
            nBytesWritten = EchoServer.Write(ioBuffer, nBytesRead)

            ' Update the client hashtable with the amount of data written
            SyncLock ClientTable.SyncRoot
                nBytesTotal = ClientTable(EchoServer.ClientId) + nBytesWritten
                ClientTable(EchoServer.ClientId) = nBytesTotal
            End SyncLock
        End If
    End Sub
End Class
