' SocketTools 10 Build 1468
' Copyright 2023 Catalyst Development Corporation
' All rights reserved
'
' This file is licensed to you pursuant to the terms of the
' product license agreement included with the original software,
' and is protected by copyright law and international treaties.
' Unauthorized reproduction or distribution may result in severe
' criminal penalties.
'
Option Strict Off
Option Explicit On

Friend Class formMain
    Inherits System.Windows.Forms.Form

    Private WithEvents httpServer As New SocketTools.HttpServer

    Delegate Sub UpdateFormCallback(ByVal bEnable As Boolean)

    Private Sub UpdateForm(ByVal bEnable As Boolean)
        If cmdStart.InvokeRequired Then
            Dim d As New UpdateFormCallback(AddressOf UpdateForm)
            Me.Invoke(d, New Object() {bEnable})
        Else
            If HttpServer.IsActive Then
                cmdStart.Enabled = bEnable
                cmdSuspend.Enabled = bEnable
                cmdRestart.Enabled = bEnable
            Else
                cmdStart.Enabled = bEnable
                cmdSuspend.Enabled = False
                cmdRestart.Enabled = False
            End If
        End If
    End Sub

    Delegate Sub UpdateStatusCallback(ByVal strMessage As String)

    Private Sub UpdateStatus(ByVal strMessage As String)
        If lblStatus.InvokeRequired Then
            Dim d As New UpdateStatusCallback(AddressOf UpdateStatus)
            Me.Invoke(d, New Object() {strMessage})
        Else
            lblStatus.Text = strMessage
            lblClient.Text = HttpServer.ClientCount & " Clients"
        End If
    End Sub

    Private Sub formMain_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load
        comboFormats.Items.Insert(0, "None")
        comboFormats.Items.Insert(1, "Common")
        comboFormats.Items.Insert(2, "Extended")
        comboFormats.SelectedIndex = 0
    End Sub

    Private Sub cmdStart_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdStart.Click
        If Not httpServer.IsActive Then

            UpdateForm(False)

            ' Basic server configuration
            httpServer.ServerName = Trim(editHostName.Text)
            httpServer.ServerAddress = Trim(editLocalAddress.Text)
            httpServer.ServerPort = CInt(Trim(editLocalPort.Text))
            httpServer.Identity = Trim(editIdentity.Text)
            httpServer.Directory = Trim(editDirectory.Text)

            ' Server limits
            httpServer.MaxClients = CInt(editMaxClients.Text)
            httpServer.IdleTime = CInt(editIdleTime.Text)

            ' Server options
            httpServer.Secure = CBool(chkSecure.CheckState)
            httpServer.LocalUser = CBool(chkLocalUser.CheckState)
            httpServer.MultiUser = CBool(chkMultiUser.CheckState)
            httpServer.LockFiles = CBool(chkLockFiles.CheckState)
            httpServer.Restricted = CBool(chkRestricted.CheckState)
            httpServer.ReadOnly = CBool(chkReadOnly.CheckState)
            httpServer.HiddenFiles = CBool(chkHidden.CheckState)

            ' Server logging
            httpServer.LogFormat = comboFormats.SelectedIndex
            httpServer.LogFile = Trim(editLogFile.Text)

            ' Start the server
            If httpServer.Start() Then
                cmdStart.Text = "Stop"
                UpdateStatus("Server accepting connections on " & httpServer.ServerAddress & ":" & httpServer.ServerPort)
                UpdateForm(True)
            Else
                MsgBox(httpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        Else
            UpdateForm(False)

            If httpServer.Stop() Then
                cmdStart.Text = "Start"
            Else
                MsgBox(httpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        End If
    End Sub

    Private Sub cmdSuspend_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdSuspend.Click
        If cmdSuspend.Text = "Suspend" Then
            UpdateForm(False)

            If httpServer.Suspend() Then
                cmdSuspend.Text = "Resume"
                UpdateStatus("Server suspended accepting new client connections")
                UpdateForm(True)
            Else
                MsgBox(httpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        Else
            UpdateForm(False)

            If httpServer.Resume() Then
                cmdSuspend.Text = "Suspend"
                UpdateStatus("Server resumed accepting new client connections")
                UpdateForm(True)
            Else
                MsgBox(httpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        End If
    End Sub

    Private Sub cmdRestart_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdRestart.Click
        UpdateForm(False)

        If httpServer.Restart() Then
            cmdSuspend.Text = "Suspend"
            UpdateStatus("The server has been restarted")
            UpdateForm(True)
        Else
            MsgBox(httpServer.LastErrorString, MsgBoxStyle.Exclamation)
            UpdateForm(True)
            Exit Sub
        End If
    End Sub

    Private Sub cmdClose_Click(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles cmdClose.Click
        If httpServer.IsActive Then
            UpdateForm(False)

            If httpServer.Stop() Then
                UpdateForm(True)
            Else
                MsgBox(httpServer.LastErrorString, MsgBoxStyle.Exclamation)
                UpdateForm(True)
                Exit Sub
            End If
        End If

        Me.Close()
    End Sub

    Private Sub httpServer_OnAuthenticate(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.AuthenticateEventArgs) Handles httpServer.OnAuthenticate
        Debug.WriteLine("Client " & eventArgs.ClientId & " authenticating as " & eventArgs.UserName)
    End Sub

    Private Sub httpServer_OnCommand(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.CommandEventArgs) Handles httpServer.OnCommand
        Debug.WriteLine("Client " & eventArgs.ClientId & " sent command " & eventArgs.Command & " [" & eventArgs.Parameters & "]")
        Debug.WriteLine("Virtual Path: " & httpServer.VirtualPath)
        Debug.WriteLine("Local Path:   " & httpServer.LocalPath)
    End Sub

    Private Sub httpServer_OnConnect(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.ConnectEventArgs) Handles httpServer.OnConnect
        Debug.WriteLine("Client " & eventArgs.ClientId & " connected from " & eventArgs.ClientAddress)
        UpdateStatus("Client " & eventArgs.ClientId & " connected from " & eventArgs.ClientAddress)
    End Sub

    Private Sub httpServer_OnDisconnect(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.DisconnectEventArgs) Handles httpServer.OnDisconnect
        Debug.WriteLine("Client " & eventArgs.ClientId & " disconnected")
        UpdateStatus("Client " & eventArgs.ClientId & " disconnected from the server")
    End Sub

    Private Sub httpServer_OnDownload(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.DownloadEventArgs) Handles httpServer.OnDownload
        Debug.WriteLine("Client " & eventArgs.ClientId & " downloaded " & eventArgs.FileName & " (" & eventArgs.FileSize & " bytes)")
    End Sub

    Private Sub httpServer_OnError(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.ErrorEventArgs) Handles httpServer.OnError
        Debug.WriteLine("Client " & eventArgs.ClientId & " error " & eventArgs.Error & ": " & eventArgs.Description)
    End Sub

    Private Sub httpServer_OnExecute(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.ExecuteEventArgs) Handles httpServer.OnExecute
        Debug.WriteLine("Client " & eventArgs.ClientId & " executed " & eventArgs.Resource & " (" & eventArgs.ExitCode & ")")
    End Sub

    Private Sub httpServer_OnIdle(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles httpServer.OnIdle
        Debug.WriteLine("All clients have disconnected, the server is idle")
    End Sub

    Private Sub httpServer_OnResult(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.ResultEventArgs) Handles httpServer.OnResult
        Debug.WriteLine("Client " & eventArgs.ClientId & " command " & eventArgs.Command & " completed (" & eventArgs.ResultCode & ")")
    End Sub

    Private Sub httpServer_OnStart(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles httpServer.OnStart
        Debug.WriteLine("Server has started accepting connections")
        UpdateStatus("Server accepting connections on " & httpServer.ServerAddress & ":" & httpServer.ServerPort)
        UpdateForm(True)
    End Sub

    Private Sub httpServer_OnStop(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles httpServer.OnStop
        Debug.WriteLine("Server has stopped, all clients terminated")
        UpdateStatus("Server stopped accepting connections")
        UpdateForm(True)
    End Sub

    Private Sub httpServer_OnTimeout(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.TimeoutEventArgs) Handles httpServer.OnTimeout
        Debug.WriteLine("Client " & eventArgs.ClientId & " timed out after " & eventArgs.Elapsed & " seconds")
    End Sub

    Private Sub httpServer_OnUpload(ByVal eventSender As System.Object, ByVal eventArgs As SocketTools.HttpServer.UploadEventArgs) Handles httpServer.OnUpload
        Debug.WriteLine("Client " & eventArgs.ClientId & " uploaded " & eventArgs.FileName & " (" & eventArgs.FileSize & " bytes)")
    End Sub
End Class
